#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <GL/glut.h>
#include <GL/glu.h>
#include <GL/gl.h>

#define	KEY_ESC	27
#define	DELTA_T	0.05

#define	BoxWidth	0.1
#define	BoxHeight	0.1
#define	BoxRemove	0.3
#define	SpringWidth	0.1
#define	SpringRolls	17
#define	GrandRemove	-0.5

double	t=0.0;
double	BoxPosition,BoxVelocity,BoxCoordinate;
double	GrandPosition,GrandVelocity,GrandCoordinate;
double	Myu,P,W,A=0.1,DampingRatio;
double	stretch,Distance=BoxRemove-GrandRemove;

void Input_Data( void );
void Initial_State( void );
void Calculate( void );
void myInit( char * );
void draw( void );
void display( void );
void move( void );
void myKbd( unsigned char, int, int );


main( int argc, char **argv )
{
	Initial_State();
	Input_Data();
	
	glutInit( &argc, argv );
	myInit( argv[0] );
	
	glutDisplayFunc( display );
	glutIdleFunc( move );
	
	glutMainLoop();
	
	return(0);
}

void myInit( char *progname )
{
	int width =500, height =500;
	glutInitWindowPosition( 0,0 );
	glutInitWindowSize( width, height );
	
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA );
	
	glutCreateWindow( progname );
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glutKeyboardFunc( myKbd );
	
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho( -1.0, 1.0, -1.0, 1.0, -1.0, 1.0 );
}

void display( void )
{
	glClear( GL_COLOR_BUFFER_BIT );
	
	Calculate();
	
	draw();
	
	glutSwapBuffers();
}

void draw( void )
{
	int	i,j=1;
	glBegin( GL_POLYGON );
		glColor3d( 1.0, 1.0, 0.5 );
		glVertex3f( -0.5, GrandCoordinate+0.05, 0.0 );
		glVertex3f( -0.5, GrandCoordinate-0.05, 0.0 );
		glVertex3f(  0.5, GrandCoordinate-0.05, 0.0 );
		glVertex3f(  0.5, GrandCoordinate+0.05, 0.0 );
	glEnd();
	
	glBegin( GL_POLYGON );
		glColor3d( 1.0, 0.0, 0.0 );
		glVertex3f( -BoxWidth, -BoxHeight+BoxCoordinate, 0.0 );
		glVertex3f( -BoxWidth,  BoxHeight+BoxCoordinate, 0.0 );
		glVertex3f(  BoxWidth,  BoxHeight+BoxCoordinate, 0.0 );
		glVertex3f(  BoxWidth, -BoxHeight+BoxCoordinate, 0.0 );
	glEnd();
	
	stretch = ((BoxCoordinate-BoxHeight-0.05)-(GrandCoordinate+0.1))/SpringRolls;
	glBegin( GL_LINE_STRIP );
		glColor3f( 1.0, 1.0, 1.0 );
		glVertex3f( 0.0, BoxCoordinate-BoxHeight, 0.0 );
		glVertex3f( 0.0, BoxCoordinate-BoxHeight-0.05, 0.0 );
		for( i=1; i<SpringRolls; i+=2)
		{
			glVertex3f( pow(-1,j)*SpringWidth, BoxCoordinate-BoxHeight-0.05-i*stretch, 0.0 );
			j++;
		}
		glVertex3f( 0.0, GrandCoordinate+0.1, 0.0 );
		glVertex3f( 0.0, GrandCoordinate+0.05, 0.0 );
	glEnd();
}

void move( void )
{
	t += DELTA_T;
	glutPostRedisplay();
}

void myKbd( unsigned char key, int x, int y )
{
	switch( key )
	{
		case KEY_ESC:
			exit( 0 );
		
		case 'r':
		case 'R':
			t=0.0;
			Initial_State();
			break;
			
		default:
			break;
	}
}

void Input_Data( void )
{
	do{
		printf("Damping Ratio=");
		scanf("%lf",&DampingRatio);
	}while( DampingRatio<0 );
	
	do{
		printf("p=");
		scanf("%lf",&P);
	}while( P<=0 );
	
	do{
		printf("w=");
		scanf("%lf",&W);
	}while( W<0 );
	Myu = DampingRatio*Myu;
}

void Initial_State( void )
{
	BoxPosition = 0.0;
	BoxVelocity = 0.0;
	GrandPosition = 0.0;
	GrandPosition = 0.0;
}

void Calculate( void )
{
	GrandPosition = A*sin(W*t);
	GrandVelocity = A*W*cos(W*t);
	BoxPosition = BoxPosition + BoxVelocity*DELTA_T;
	BoxVelocity = BoxVelocity +(-2*Myu*(BoxVelocity-GrandVelocity) -P*P*(BoxPosition-GrandPosition))*DELTA_T;
	GrandCoordinate = GrandPosition +GrandRemove;
	BoxCoordinate = BoxPosition +BoxRemove;
}